// Author: M.Gheata  06/16/03
void geoshapes ()
{
// root[0] .x geoshapes.C
// root[1] box();   //draw a TGeoBBox with description
//
// The box can be divided on one axis.
//
// root[2] box(iaxis, ndiv, start, step);
//
// where: iaxis = 1,2 or 3, meaning (X,Y,Z) or (Rxy, phi, Z) depending on shape type
//        ndiv  = number of slices
//        start = starting position (must be in shape range)
//        step  = division step
// If step=0, all range of a given axis will be divided
// 
// The same can procedure can be performed for visualizing other shapes.
// When drawing one shape after another, the old geometry/canvas will be deleted.
   gSystem->Load("libGeom");
   bar = new TControlBar("vertical", "TGeo shapes",10,10);
   bar->AddButton(" How to run ","help()","Instructions for running this macro"); 
   bar->AddButton("Box         ","box()","A box shape.");
   bar->AddButton("Tube        ","tube()","A tube with inner and outer radius");
   bar->AddButton("Tube segment","tubeseg()","A tube segment");
   bar->AddButton("Cut tube    ","ctub()","A cut tube segment");
   bar->AddButton("Cone        ","cone()","A conical tube");
   bar->AddButton("Cone segment","coneseg()","A conical segment");
   bar->AddButton("Sphere      ","sphere()","A spherical sector");
   bar->AddButton("Eliptical tube","eltu()","An eliptical tube");
   bar->AddButton("Polygone    ","pgon()","A polygone"); 
   bar->AddButton("Polycone    ","pcon()","A polycone shape");
   bar->AddButton("Twisted trap","gtra()","A twisted trapezoid");
   bar->AddButton("Arb8        ","arb8()","An arbitrary polyedron defined by vertices (max 8) sitting on 2 parallel planes");
   bar->AddButton("Parallelipiped","para()","A parallelipiped shape");
   bar->AddButton("Trd1","trd1()","A trapezoid with dX varying with Z");
   bar->AddButton("Trd2","trd2()","A trapezoid with both dX and dY varying with Z");
   bar->AddButton("Trapezoid","trap()","A general trapezoid");
   bar->Show();
   gROOT->SaveContext();
}
void box(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   TCanvas *c = new TCanvas("box shape", "A simple box", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("box", "poza1");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeBox("BOX",med, 20,30,40);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoBBox *box = (TGeoBBox*)(vol->GetShape());
   TText *text = pt->AddText("TGeoBBox - box class");
   text->SetTextColor(2);
   AddText(pt,"fDX",box->GetDX(),"half length in X");
   AddText(pt,"fDY",box->GetDY(),"half length in Y");
   AddText(pt,"fDZ",box->GetDZ(),"half length in Z");
   AddText(pt,"fOrigin[0]",(box->GetOrigin())[0],"box origin on X");
   AddText(pt,"fOrigin[1]",(box->GetOrigin())[1],"box origin on Y");
   AddText(pt,"fOrigin[2]",(box->GetOrigin())[2],"box origin on Z");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: box(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 1, 2 or 3 (X, Y, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetTextSize(0.044);
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->Draw();
//   SavePicture("box",c,vol,iaxis,step);
   c->cd(1);
}

void para(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   TCanvas *c = new TCanvas("para shape", "A parallelipiped", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("para", "poza1");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakePara("PARA",med, 20,30,40,30,15,30);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoPara *para = (TGeoPara*)(vol->GetShape());
   TText *text = pt->AddText("TGeoPara - parallelipiped class");
   text->SetTextColor(2);
   AddText(pt,"fX",para->GetX(),"half length in X");
   AddText(pt,"fY",para->GetY(),"half length in Y");
   AddText(pt,"fZ",para->GetZ(),"half length in Z");
   AddText(pt,"fAlpha",para->GetAlpha(),"angle about Y of the Z bases");
   AddText(pt,"fTheta",para->GetTheta(),"inclination of para axis about Z");
   AddText(pt,"fPhi",para->GetPhi(),"phi angle of para axis");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: para(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 1, 2 or 3 (X, Y, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetTextSize(0.044);
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->Draw();
//   SavePicture("para",c,vol,iaxis,step);
}

void tube(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   TCanvas *c = new TCanvas("tube shape", "A tube", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("tube", "poza2");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeTube("TUBE",med, 20,30,40);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTube *tube = (TGeoTube*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTube - tube class");
   text->SetTextColor(2);
   AddText(pt,"fRmin",tube->GetRmin(),"minimum radius");
   AddText(pt,"fRmax",tube->GetRmax(),"maximum radius");
   AddText(pt,"fDZ",  tube->GetDZ(),  "half length in Z");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: tube(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 1, 2 or 3 (Rxy, Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("tube",c,vol,iaxis,step);
}

void tubeseg(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   TCanvas *c = new TCanvas("tubeseg shape", "A tube segment ", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("tubeseg", "poza3");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeTubs("TUBESEG",med, 20,30,40,-30,250);
   vol->SetLineColor(4);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTubeSeg *tubeseg = (TGeoTubeSeg*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTubeSeg - tube segment class");
   text->SetTextColor(2);
   AddText(pt,"fRmin",tubeseg->GetRmin(),"minimum radius");
   AddText(pt,"fRmax",tubeseg->GetRmax(),"maximum radius");
   AddText(pt,"fDZ",  tubeseg->GetDZ(),  "half length in Z");
   AddText(pt,"fPhi1",tubeseg->GetPhi1(),"first phi limit");
   AddText(pt,"fPhi2",tubeseg->GetPhi2(),"second phi limit");  
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: tubeseg(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 1, 2 or 3 (Rxy, Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("tubeseg",c,vol,iaxis,step);
}

void ctub(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>2) {
      printf("Wrong division axis. Range is 1-2.\n");
      return;
   }   
   TCanvas *c = new TCanvas("ctub shape", "A cut tube segment ", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("ctub", "poza3");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   Double_t theta = 160.*TMath::Pi()/180.;
   Double_t phi   = 30.*TMath::Pi()/180.;
   Double_t nlow[3];
   nlow[0] = TMath::Sin(theta)*TMath::Cos(phi);
   nlow[1] = TMath::Sin(theta)*TMath::Sin(phi);
   nlow[2] = TMath::Cos(theta);
   theta = 20.*TMath::Pi()/180.;
   phi   = 60.*TMath::Pi()/180.;
   Double_t nhi[3];
   nhi[0] = TMath::Sin(theta)*TMath::Cos(phi);
   nhi[1] = TMath::Sin(theta)*TMath::Sin(phi);
   nhi[2] = TMath::Cos(theta);
   TGeoVolume *vol = gGeoManager->MakeCtub("CTUB",med, 20,30,40,-30,250, nlow[0], nlow[1], nlow[2], nhi[0],nhi[1],nhi[2]);
   vol->SetLineColor(4);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTubeSeg *tubeseg = (TGeoTubeSeg*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTubeSeg - tube segment class");
   text->SetTextColor(2);
   AddText(pt,"fRmin",tubeseg->GetRmin(),"minimum radius");
   AddText(pt,"fRmax",tubeseg->GetRmax(),"maximum radius");
   AddText(pt,"fDZ",  tubeseg->GetDZ(),  "half length in Z");
   AddText(pt,"fPhi1",tubeseg->GetPhi1(),"first phi limit");
   AddText(pt,"fPhi2",tubeseg->GetPhi2(),"second phi limit");  
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText(" ");
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("tubeseg",c,vol,iaxis,step);
}
void cone(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   if (iaxis==1) {
      printf("cannot divide cone on Rxy\n");
      return;
   }   
   TCanvas *c = new TCanvas("cone shape", "A cone", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("cone", "poza4");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeCone("CONE",med, 40,30,40,35,45);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   top->AddNode(vol,1);
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(20);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoCone *cone = (TGeoCone*)(vol->GetShape());
   TText *text = pt->AddText("TGeoCone - cone class");
   text->SetTextColor(2);
   AddText(pt,"fDZ",  cone->GetDZ(),    "half length in Z");
   AddText(pt,"fRmin1",cone->GetRmin1(),"inner radius at -dz");
   AddText(pt,"fRmax1",cone->GetRmax1(),"outer radius at -dz");
   AddText(pt,"fRmin2",cone->GetRmin2(),"inner radius at +dz");
   AddText(pt,"fRmax2",cone->GetRmax2(),"outer radius at +dz");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: cone(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("cone",c,vol,iaxis,step);
}

void coneseg(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   TCanvas *c = new TCanvas("coneseg shape", "A cone segment", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("coneseg", "poza5");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeCons("CONESEG",med, 40,30,40,10,20,-30,250);
   vol->SetLineColor(4);
//   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoConeSeg *coneseg = (TGeoConeSeg*)(vol->GetShape());
   TText *text = pt->AddText("TGeoConeSeg - coneseg class");
   text->SetTextColor(2);
   AddText(pt,"fDZ",  coneseg->GetDZ(),    "half length in Z");
   AddText(pt,"fRmin1",coneseg->GetRmin1(),"inner radius at -dz");
   AddText(pt,"fRmax1",coneseg->GetRmax1(),"outer radius at -dz");
   AddText(pt,"fRmin2",coneseg->GetRmin1(),"inner radius at +dz");
   AddText(pt,"fRmax2",coneseg->GetRmax1(),"outer radius at +dz");
   AddText(pt,"fPhi1",coneseg->GetPhi1(),"first phi limit");
   AddText(pt,"fPhi2",coneseg->GetPhi2(),"second phi limit"); 
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: coneseg(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("coneseg",c,vol,iaxis,step);
}

void eltu(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   TCanvas *c = new TCanvas("eltu shape", "An Elliptical tube", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("eltu", "poza6");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeEltu("ELTU",med, 30,10,40);
   vol->SetLineColor(4);
//   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoEltu *eltu = (TGeoEltu*)(vol->GetShape());
   TText *text = pt->AddText("TGeoEltu - eltu class");
   text->SetTextColor(2);
   AddText(pt,"fA",eltu->GetA(), "semi-axis along x");
   AddText(pt,"fB",eltu->GetB(), "semi-axis along y");
   AddText(pt,"fDZ", eltu->GetDZ(),  "half length in Z");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: eltu(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("eltu",c,vol,iaxis,step);
}

void sphere(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis!=0) {
      printf("Cannot divide spheres\n");
      return;
   }   
   TCanvas *c = new TCanvas("sphere shape", "A spherical sector", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("sphere", "poza7");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeSphere("SPHERE",med, 30,40,60,120,30,240);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   gGeoManager->SetNsegments(40);
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoSphere *sphere = (TGeoSphere*)(vol->GetShape());
   TText *text = pt->AddText("TGeoSphere- sphere class");
   text->SetTextColor(2);
   AddText(pt,"fRmin",sphere->GetRmin(),"inner radius");
   AddText(pt,"fRmax",sphere->GetRmax(),"outer radius");
   AddText(pt,"fTheta1",sphere->GetTheta1(),"lower theta limit");
   AddText(pt,"fTheta2",sphere->GetTheta2(),"higher theta limit");
   AddText(pt,"fPhi1",sphere->GetPhi1(),"lower phi limit");
   AddText(pt,"fPhi2",sphere->GetPhi2(),"higher phi limit"); 
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText(" ");
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("sphere",c,vol,iaxis,step);
}

void trd1(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   if (iaxis==1) {
      printf("Cannot divide trd1 on X axis\n");
      return;
   }   
   
   TCanvas *c = new TCanvas("trd1 shape", "A trapezoid with dX varying", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("trd1", "poza8");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeTrd1("Trd1",med, 10,20,30,40);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTrd1 *trd1 = (TGeoTrd1*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTrd1 - Trd1 class");
   text->SetTextColor(2);
   AddText(pt,"fDx1",trd1->GetDx1(),"half length in X at lower Z surface(-dz)");
   AddText(pt,"fDx2",trd1->GetDx2(),"half length in X at higher Z surface(+dz)");
   AddText(pt,"fDy",trd1->GetDy(),"half length in Y");
   AddText(pt,"fDz",trd1->GetDz(),"half length in Z");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: trd1(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Y, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("trd1",c,vol,iaxis,step);
}

void pcon(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   if (iaxis==1) {
      printf("Cannot divide pcon on Rxy\n");
      return;
   }   
   TCanvas *c = new TCanvas("pcon shape", "A polycone", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("pcon", "poza10");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakePcon("PCON",med, -30.0,300,4);
   TGeoPcon *pcon = (TGeoPcon*)(vol->GetShape());
   pcon->DefineSection(0,0,15,20);
   pcon->DefineSection(1,20,15,20);
   pcon->DefineSection(2,20,15,25);
   pcon->DefineSection(3,50,15,20);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TText *text = pt->AddText("TGeoPcon - pcon class");
   text->SetTextColor(2);
   AddText(pt,"fPhi1",pcon->GetPhi1(),"lower phi limit");
   AddText(pt,"fDphi",pcon->GetDphi(),"phi range");
   AddText(pt,"fNz",pcon->GetNz(),"number of z planes");
   for (Int_t j=0; j<pcon->GetNz(); j++) {
      char line[128];
      sprintf(line, "fZ[%i]=%5.2f  fRmin[%i]=%5.2f  fRmax[%i]=%5.2f", 
              j,pcon->GetZ()[j],j,pcon->GetRmin()[j],j,pcon->GetRmax()[j]);
      text = pt->AddText(line);
      text->SetTextColor(4);
      text->SetTextAlign(12);
   }   
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: pcon(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("pcon",c,vol,iaxis,step);
}

void pgon(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis<0 || iaxis>3) {
      printf("Wrong division axis. Range is 1-3.\n");
      return;
   }   
   if (iaxis==1) {
      printf("Cannot divide pgon on Rxy\n");
      return;
   }   
   TCanvas *c = new TCanvas("pgon shape", "A polygone", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("pgon", "poza11");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakePgon("PGON",med, -45.0,270.0,4,2);
   TGeoPgon *pgon = (TGeoPgon*)(vol->GetShape());
   pgon->DefineSection(0,-70,25,50);
   pgon->DefineSection(1,70,50,100);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TText *text = pt->AddText("TGeoPgon - pgon class");
   text->SetTextColor(2);
   AddText(pt,"fPhi1",pgon->GetPhi1(),"lower phi limit");
   AddText(pt,"fDphi",pgon->GetDphi(),"phi range");
   AddText(pt,"fNedges",pgon->GetNedges(),"number of edges");
    AddText(pt,"fNz",pgon->GetNz(),"number of z planes");
   for (Int_t j=0; j<pgon->GetNz(); j++) {
      char line[128];
      sprintf(line, "fZ[%i]=%5.2f  fRmin[%i]=%5.2f  fRmax[%i]=%5.2f", 
              j,pgon->GetZ()[j],j,pgon->GetRmin()[j],j,pgon->GetRmax()[j]);
      text = pt->AddText(line);
      text->SetTextColor(4);
      text->SetTextAlign(12);
   }   
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: pgon(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be 2 or 3 (Phi, Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("pgon",c,vol,iaxis,step);
}

void arb8(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis!=0) {
      printf("Cannot divide arb8\n");
      return;
   }   
   TCanvas *c = new TCanvas("arb8 shape", "An arbitrary polyedron", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("arb8", "poza12");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoArb8 *arb = new TGeoArb8(20);
   arb->SetVertex(0,-30,-25);
   arb->SetVertex(1,-25,25);
   arb->SetVertex(2,5,25);
   arb->SetVertex(3,25,-25);
   arb->SetVertex(4,-28,-23);
   arb->SetVertex(5,-23,27);
   arb->SetVertex(6,-23,27);
   arb->SetVertex(7,13,-27);
   TGeoVolume *vol = new TGeoVolume("ARB8",arb,med);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TText *text = pt->AddText("TGeoArb8 - arb8 class");
   text->SetTextColor(2);
   AddText(pt,"fDz",arb->GetDz(),"Z half length");
   char line[128];
   Double_t *vert = arb->GetVertices();
   text = pt->AddText("Vertices on lower Z plane:");
   text->SetTextColor(3);
   Int_t i;
   for (i=0; i<4; i++) {
      sprintf(line,"   fXY[%d] = (%5.2f, %5.2f)", i, vert[2*i], vert[2*i+1]);
      text = pt->AddText(line);
      text->SetTextSize(0.043);
      text->SetTextColor(4);
   }      
   text = pt->AddText("Vertices on higher Z plane:");
   text->SetTextColor(3);
   for (i=4; i<8; i++) {
      sprintf(line,"   fXY[%d] = (%5.2f, %5.2f)", i, vert[2*i], vert[2*i+1]);
      text = pt->AddText(line);
      text->SetTextSize(0.043);
      text->SetTextColor(4);
   }      
      
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText(" ");
   pt->SetTextSize(0.043);
   pt->Draw();
//   SavePicture("arb8",c,vol,iaxis,step);
}

void trd2(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis && iaxis!=3) {
      printf("Wrong division axis. Can divide only in Z (3)\n");
      return;
   }   
   TCanvas *c = new TCanvas("trd2 shape", "A trapezoid with dX and dY varying with Z", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("trd2", "poza9");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeTrd2("Trd2",med, 10,20,30,10,40);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTrd2 *trd2 = (TGeoTrd2*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTrd2 - Trd2 class");
   text->SetTextColor(2);
   AddText(pt,"fDx1",trd2->GetDx1(),"half length in X at lower Z surface(-dz)");
   AddText(pt,"fDx2",trd2->GetDx2(),"half length in X at higher Z surface(+dz)");
   AddText(pt,"fDy1",trd2->GetDy1(),"half length in Y at lower Z surface(-dz)");
   AddText(pt,"fDy2",trd2->GetDy2(),"half length in Y at higher Z surface(-dz)");
   AddText(pt,"fDz",trd2->GetDz(),"half length in Z");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: trd2(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be only 3 (Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("trd2",c,vol,iaxis,step);
}

void trap(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis && iaxis!=3) {
      printf("Wrong division axis. Can divide only in Z (3)\n");
      return;
   }   
   TCanvas *c = new TCanvas("trap shape", "A more general trapezoid", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("trap", "poza10");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeTrap("Trap",med, 30,15,30,20,10,15,0,20,10,15,0);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoTrap *trap = (TGeoTrap*)(vol->GetShape());
   TText *text = pt->AddText("TGeoTrap - Trapezoid class");
   text->SetTextColor(2);
   AddText(pt,"fDz",trap->GetDz(),"half length in Z");
   AddText(pt,"fTheta",trap->GetTheta(),"theta angle of trapezoid axis");
   AddText(pt,"fPhi",trap->GetPhi(),"phi angle of trapezoid axis");
   AddText(pt,"fH1",trap->GetH1(),"half length in y at -fDz");
   AddText(pt,"fAlpha1",trap->GetAlpha1(),"angle between centers of x edges and y axis at -fDz");
   AddText(pt,"fBl1",trap->GetBl1(),"half length in x at -dZ and y=-fH1");
   AddText(pt,"fTl1",trap->GetTl1(),"half length in x at -dZ and y=+fH1");
   AddText(pt,"fH2",trap->GetH2(),"half length in y at +fDz");
   AddText(pt,"fBl2",trap->GetBl2(),"half length in x at +dZ and y=-fH1");
   AddText(pt,"fTl2",trap->GetTl2(),"half length in x at +dZ and y=+fH1");
   AddText(pt,"fAlpha2",trap->GetAlpha2(),"angle between centers of x edges and y axis at +fDz");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: trap(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be only 3 (Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("trap",c,vol,iaxis,step);
}

void gtra(Int_t iaxis=0, Int_t ndiv=8, Double_t start=0, Double_t step=0)
{
   gROOT->GetListOfCanvases()->Delete();
   if (iaxis && iaxis!=3) {
      printf("Wrong division axis. Can divide only in Z (3)\n");
      return;
   }   
   TCanvas *c = new TCanvas("gtra shape", "A twisted trapezoid", 700,800);
   c->Divide(1,2,0,0);
   c->cd(1);
   if (gGeoManager) delete gGeoManager;
   new TGeoManager("gtra", "poza11");
   TGeoMaterial *mat = new TGeoMaterial("VOID",0,0,0);
   TGeoMedium *med = new TGeoMedium("MED",1,mat);
   TGeoVolume *top = gGeoManager->MakeBox("TOP",med,100,100,100);
   gGeoManager->SetTopVolume(top);
   TGeoVolume *vol = gGeoManager->MakeGtra("Gtra",med, 30,15,30,30,20,10,15,0,20,10,15,0);
   vol->SetLineColor(4);
   vol->SetLineWidth(2);
   top->AddNode(vol,1);
   if (iaxis) {
      TGeoVolume *slice = vol->Divide("SLICE",iaxis,ndiv,start,step);
      if (!slice) return;
      slice->SetLineColor(2);
   }
   gGeoManager->CloseGeometry();
   top->Draw();
   MakePicture();
   c->cd(2);
   TPaveText *pt = new TPaveText(0.01,0.01,0.99,0.99);
   pt->SetLineColor(1);
   TGeoGtra *trap = (TGeoGtra*)(vol->GetShape());
   TText *text = pt->AddText("TGeoGtra - Twisted trapezoid class");
   text->SetTextColor(2);
   AddText(pt,"fDz",trap->GetDz(),"half length in Z");
   AddText(pt,"fTheta",trap->GetTheta(),"theta angle of trapezoid axis");
   AddText(pt,"fPhi",trap->GetPhi(),"phi angle of trapezoid axis");
   AddText(pt,"fTwist",trap->GetTwistAngle(), "twist angle");
   AddText(pt,"fH1",trap->GetH1(),"half length in y at -fDz");
   AddText(pt,"fAlpha1",trap->GetAlpha1(),"angle between centers of x edges and y axis at -fDz");
   AddText(pt,"fBl1",trap->GetBl1(),"half length in x at -dZ and y=-fH1");
   AddText(pt,"fTl1",trap->GetTl1(),"half length in x at -dZ and y=+fH1");
   AddText(pt,"fH2",trap->GetH2(),"half length in y at +fDz");
   AddText(pt,"fBl2",trap->GetBl2(),"half length in x at +dZ and y=-fH1");
   AddText(pt,"fTl2",trap->GetTl2(),"half length in x at +dZ and y=+fH1");
   AddText(pt,"fAlpha2",trap->GetAlpha2(),"angle between centers of x edges and y axis at +fDz");
   if (iaxis) AddText(pt, vol->GetFinder(), iaxis);
   pt->AddText("Execute: gtra(iaxis, ndiv, start, step) to divide this.");
   pt->AddText("----- IAXIS can be only 3 (Z)");
   pt->AddText("----- NDIV must be a positive integer");
   pt->AddText("----- START must be a valid axis offset within shape range on divided axis");
   pt->AddText("----- STEP is the division step. START+NDIV*STEP must be in range also");
   pt->AddText("----- If START and STEP are omitted, all range of the axis will be divided");
   pt->AddText(" ");
   pt->SetAllWith("-----","color",2);
   pt->SetAllWith("-----","font",72);
   pt->SetAllWith("-----","size",0.04);
   pt->SetAllWith("Execute","color",4);
   pt->SetTextAlign(12);
   pt->SetTextSize(0.044);
   pt->Draw();
//   SavePicture("gtra",c,vol,iaxis,step);
}

void MakePicture()
{
   gGeoManager->SetNsegments(80);
   TView *view = gPad->GetView();
   if (view) {
      view->RotateView(228,86);
      view->ShowAxis();
      view->MoveViewCommand('k',1);
   }      
}
   
   
void AddText(TPaveText *pave, const char *datamember, Double_t value, const char *comment)
{
   char line[128];
   for (Int_t i=0; i<128; i++) line[i] = ' ';
   memcpy(&line[0], datamember, strlen(datamember));
   line[10] = '=';
   char number[20];
   sprintf(number, "%5.2f", value);
   memcpy(&line[12], number, strlen(number));
   line[26] = '=';
   line[27] = '>';
   sprintf(&line[30], "%s",comment);
   TText *text = pave->AddText(line);
//   text->SetTextColor(4);
   text->SetTextAlign(12);//12
}

void AddText(TPaveText *pave, const char *datamember, Int_t value, const char *comment)
{
   char line[128];
   for (Int_t i=0; i<128; i++) line[i] = ' ';
   memcpy(&line[0], datamember, strlen(datamember));
   line[10] = '=';
   char number[20];
   sprintf(number, "%5i", value);
   memcpy(&line[12], number, strlen(number));
   line[26] = '=';
   line[27] = '>';
   sprintf(&line[30], "%s",comment);
   TText *text = pave->AddText(line);
//   text->SetTextColor(4);
   text->SetTextAlign(12);
}

void AddText(TPaveText *pave, TObject *pf, Int_t iaxis)
{
   char line[128];
   TGeoPatternFinder *finder = (TGeoPatternFinder*)pf;
   if (!pave || !pf) return;
   for (Int_t i=0; i<128; i++) line[i] = ' ';
   TGeoVolume *volume = finder->GetVolume();
   TGeoShape *sh = volume->GetShape();
   sprintf(line, "Division of %s on axis %d (%s)", volume->GetName(), iaxis,sh->GetAxisName(iaxis));
   TText *text = pave->AddText(line);
   text->SetTextColor(3);
   text->SetTextAlign(12);
   AddText(pave, "fNdiv",finder->GetNdiv(),"number of divisions");
   AddText(pave, "fStart",finder->GetStart(),"start divisioning position");
   AddText(pave, "fStep",finder->GetStep(),"division step");
}

void SavePicture(const char *name, TObject *objcanvas, TObject *objvol, Double_t iaxis, Double_t step)
{
   TCanvas *c = (TCanvas*)objcanvas;
   TGeoVolume *vol = (TGeoVolume*)objvol;
   if (!c || !vol) return;
   c->cd();
   char fname[32];
   switch (iaxis) {
      case 0:
         sprintf(fname,"t_%s.gif",name);
	 break;
      default:
         if (step==0) sprintf(fname,"t_%sdiv%s.gif", name,vol->GetShape()->GetAxisName(iaxis));
         else sprintf(fname,"t_%sdivstep%s.gif", name,vol->GetShape()->GetAxisName(iaxis));
   }	 
   c->Print(fname);
}

void help() {
   //
  
   new TCanvas("chelp","Help to run demos",200,10,700,500);

   welcome = new TPaveText(.1,.8,.9,.97);
   welcome->AddText("Welcome to the new geometry package");
   welcome->SetTextFont(32);
   welcome->SetTextColor(4);
   welcome->SetFillColor(24);
   welcome->Draw();

   hdemo = new TPaveText(.05,.05,.95,.7);
   hdemo->SetTextAlign(12);
   hdemo->SetTextFont(52);
   hdemo->AddText("- Demo for building TGeo basic shapes. Shape parameters are");
   hdemo->AddText("  displayed in the right pad");
   hdemo->AddText("- Click left mouse button to execute one demo");
   hdemo->AddText("- Execute box(1,8) to divide a box in 8 equal slices along X");
   hdemo->AddText("- Most shapes can be divided on X,Y,Z,Rxy or Phi :");
   hdemo->AddText("- .... root[0] <shape>(IAXIS, NDIV, START, STEP);");
   hdemo->AddText("  .... IAXIS = 1,2,3 meaning (X,Y,Z) or (Rxy, Phi, Z)");
   hdemo->AddText("  .... NDIV  = number of slices");
   hdemo->AddText("  .... START = start slicing position");
   hdemo->AddText("  .... STEP  = division step");
   hdemo->AddText(" ");
   hdemo->SetAllWith("....","color",2);
   hdemo->SetAllWith("....","font",72);
   hdemo->SetAllWith("....","size",0.04);

   hdemo->Draw();
}
